from plotnine import *
from datetime import datetime, timedelta
import pandas as pd
import pandas_ta as ta
import plotly.graph_objects as go
import plotly.express as px
from scipy.signal import argrelextrema
import numpy as np
import requests
import json

class GoldHand:
    def __init__(self, ticker):
        self.ticker = ticker
        self.df = None


    def download_historical_data(self):
        # Download historical stock data for the last year
        self.df = pd.DataFrame().ta.ticker(self.ticker).reset_index()
        self.df.columns = self.df.columns.str.lower()
        self.df['date']= [x.date() for x in self.df['date']]

        # Rsi
        self.df['rsi'] = ta.rsi(self.df['close'], 14)

        # SMAS
        self.df['sma_50']= ta.sma(self.df['close'], 50)
        self.df['diff_sma50'] = (self.df['close']/self.df['sma_50'] -1)*100
        self.df['sma_100']= ta.sma(self.df['close'], 100)
        self.df['diff_sma100'] = (self.df['close']/self.df['sma_100'] -1)*100
        self.df['sma_200']= ta.sma(self.df['close'], 200)
        self.df['diff_sma200'] = (self.df['close']/self.df['sma_200'] -1)*100

        #Bolinger bands
        bb = ta.bbands(self.df['close'])
        bb.columns = ['bb_lower', 'bb_mid', 'bb_upper', 'bandwidth', 'percent']
        self.df['bb_lower'] = bb['bb_lower']
        self.df['bb_upper'] = bb['bb_upper']
        self.df['diff_upper_bb'] = (self.df['bb_upper']/self.df['close'] -1)*100
        self.df['diff_lower_bb'] = (self.df['bb_lower']/self.df['close'] -1)*100

        #local min maxs
        self.df['local'] = ''
        self.df['local_text'] = ''
        max_ids = list(argrelextrema(self.df['high'].values, np.greater, order=30)[0])
        min_ids = list(argrelextrema(self.df['low'].values, np.less, order=30)[0])
        self.df.loc[min_ids, 'local'] = 'minimum'
        self.df.loc[max_ids, 'local'] = 'maximum'


        states = self.df[self.df['local']!='']['local'].index.to_list()
        problem = []
        problem_list = []
        for i in range(0, (len(states)-1) ):

            if (self.df.loc[states[i], 'local'] != self.df.loc[states[i+1], 'local']):
                if (len(problem)==0):
                    continue
                else:
                    problem.append(states[i])
                    text = self.df.loc[states[i], 'local']
                    if(text=='minimum'):
                        real_min = self.df.loc[problem, 'low'].idxmin()
                        problem.remove(real_min)
                        self.df.loc[problem, 'local']=''
                    else:
                        real_max = self.df.loc[problem, 'high'].idxmax()
                        problem.remove(real_max)
                        self.df.loc[problem, 'local']=''

                    problem = []
            else:
                problem.append(states[i])

        states = self.df[self.df['local']!='']['local'].index.to_list()

        # if first is min ad the price
        if self.df.loc[states[0], 'local']== 'minimum':
            self.df.loc[states[0],'local_text'] = f"${round(self.df.loc[states[0], 'low'], 2)}"
        else:
            self.df.loc[states[0],'local_text'] = f"${round(self.df.loc[states[0], 'high'], 2)}"

        # add last fall if last local is max
        if list(self.df[self.df['local']!='']['local'])[-1]=='maximum':
            last_min_id = self.df.loc[self.df['low']==min(self.df['low'][-3:] )].index.to_list()[0]
            self.df.loc[last_min_id , 'local'] = 'minimum'

        states = self.df[self.df['local']!='']['local'].index.to_list()


        for i in range(1,len(states)):
            prev = self.df.loc[states[i-1], 'local']
            current= self.df.loc[states[i], 'local']
            prev_high = self.df.loc[states[i-1], 'high']
            prev_low = self.df.loc[states[i-1], 'low']
            current_high = self.df.loc[states[i], 'high']
            current_low = self.df.loc[states[i], 'low']
            if current == 'maximum':
                # rise
                rise = (current_high/ prev_low -1)*100
                if rise>100:
                    self.df.loc[states[i], 'local_text'] = f'🚀🌌{round(((rise+100)/100), 2)}x<br>${round(current_high, 2)}'
                else:
                    self.df.loc[states[i], 'local_text'] = f'🚀{round(rise, 2)}%<br>${round(current_high, 2)}'
            else:
                fall = round((1-(current_low / prev_high))*100, 2)
                self.df.loc[states[i], 'local_text'] = f'🔻{fall}%<br>${round(current_low, 2)}'


    def plotly_last_year(self):
        tdf = self.df.tail(500)

        fig = go.Figure(data=go.Ohlc(x=tdf['date'], open=tdf['open'], high=tdf['high'], low=tdf['low'],close=tdf['close']))

        for index, row in tdf[tdf['local']!=''].iterrows():
            direction = row['local']
            tdate = row['date']
            local_text = row['local_text']
            min_price = row['low']
            max_price = row['high']
            if direction == 'maximum':
                fig.add_annotation( x=tdate, y=max_price, text=local_text, showarrow=True,
                align="center", bordercolor="#c7c7c7",
                font=dict(family="Courier New, monospace", size=16, color="#214e34" ), borderwidth=2,
                borderpad=4,
                bgcolor="#f4fdff",
                opacity=0.8,
                arrowhead=2,
                arrowsize=1,
                arrowwidth=1,
                ax=-45,ay=-45)

            if direction == 'minimum':
                fig.add_annotation( x=tdate, y=min_price, text=local_text, showarrow=True,
                align="center", bordercolor="#c7c7c7",
                font=dict(family="Courier New, monospace", size=16, color="red" ), borderwidth=2,
                borderpad=4,
                bgcolor="#f4fdff",
                opacity=0.8,
                arrowhead=2,
                arrowsize=1,
                arrowwidth=1,
                ax=45,ay=45)

            fig.update_layout(showlegend=False, plot_bgcolor='white', height=900, title= tw.get_plotly_title(ticker))

        fig.update_xaxes(
            mirror=True,
            ticks='outside',
            showline=True,
            linecolor='black',
            gridcolor='lightgrey'
        )
        fig.update_yaxes(
            mirror=True,
            ticks='outside',
            showline=True,
            linecolor='black',
            gridcolor='lightgrey'
        )
        fig.update(layout_xaxis_rangeslider_visible=False)
        fig.show()


